-- ***********************************************
-- ** Reticle Settings							**
-- ** Author: Divona Stormsword					**
-- ** Version 1.5.2b							**
-- ***********************************************
-- ** Special Thanks: 							**
-- ** EnSabahNur								**
-- ** Kith (Azurah)								**
-- ***********************************************

-- Getting the library from LibStub
local LAM = LibStub("LibAddonMenu-1.0")
local LMP = LibStub("LibMediaProvider-1.0")

-- ***********************************************
-- ** Variables									**
-- ***********************************************

local ReticleSettings = {}
local settings = {}

local MeleeRangeDistance = 2.5
local ActiveWeapon = GetActiveWeaponPairInfo()

local RS_ReticleContainer = false

local interactTextAltPosition = false;
local currentReticle = "Default"

-- OPTIONS: Reticles
local RETICLE_OPTIONS = {
	"Nothing", "Default", "Cross", "Crosshair", "Dot", "Midnight Squadron", "Nuke", "Round",
	"Round Square", "Custom 1", "Custom 2", "Custom 3"
}
-- OPTIONS: Labels
local LABEL_OPTIONS = {
	"Nothing", "Target Name", "Target Name + Caption", "Target Name + Level", "Target Name + Level + Caption",
	"Target Name + Level + HP",	"Target HP (Current/Max)", "Target HP (Current/Max/Percent)", "Target HP (Percent)",
	"Target Caption", "Target Distance", "Active Weapon"
}
-- DATABASE: Custom Reticle file path and size
local RETICLE_TEXTURES = {
	["Cross"] = {
		texturePath = "ReticleSettings\\Textures\\reticleCross.dds",
		textureSize = 64
	},
	["Crosshair"] = {
		texturePath = "ReticleSettings\\Textures\\reticleCrosshair.dds",
		textureSize = 64
	},
	["Dot"] = {
		texturePath = "ReticleSettings\\Textures\\reticleDot.dds",
		textureSize = 32
	},
	["Midnight Squadron"] = {
		texturePath = "ReticleSettings\\Textures\\reticleMS.dds",
		textureSize = 64
	},
	["Nuke"] = {
		texturePath = "ReticleSettings\\Textures\\reticleNuke.dds",
		textureSize = 64
	},
	["Round"] = {
		texturePath = "ReticleSettings\\Textures\\reticleRound.dds",
		textureSize = 32
	},
	["Round Square"] = {
		texturePath = "ReticleSettings\\Textures\\reticleRoundSquare.dds",
		textureSize = 64
	},
	["Custom 1"] = {
		texturePath = "ReticleSettings\\Textures\\custom1.dds",
		textureSize = 64
	},
	["Custom 2"] = {
		texturePath = "ReticleSettings\\Textures\\custom2.dds",
		textureSize = 64
	},
	["Custom 3"] = {
		texturePath = "ReticleSettings\\Textures\\custom3.dds",
		textureSize = 64
	}
}

-- Use this function to create container to put in custom Reticle.
local function createCustomReticle()

	RS_ReticleContainer = WINDOW_MANAGER:CreateControl("customReticle", ZO_ReticleContainer, CT_TEXTURE)
	RS_ReticleContainer:ClearAnchors()
	RS_ReticleContainer:SetAnchor( CENTER, ZO_ReticleContainer, CENTER, 0, 0 )
	
end

-- Use this function to load Reticle's texture
local function loadCustomReticle( reticleName )

	if ( currentReticle ~= reticleName ) then
	
		if ( reticleName == "Nothing" ) then
			currentReticle = "Nothing"
			return
		end
		
		if ( reticleName ~= "Default" ) then
	
			local rName = reticleName
			local rPath = RETICLE_TEXTURES[reticleName].texturePath
			local rSize = RETICLE_TEXTURES[reticleName].textureSize
			
			RS_ReticleContainer:SetTexture( rPath )
			RS_ReticleContainer:SetDimensions( rSize, rSize )
			currentReticle = rName
			
		else currentReticle = "Default" end
		
	end
	
end

-- ***********************************************
-- ** Update									**
-- ***********************************************

-- This function will be process every frame in-game to keep all information up-to-date.
function ReticleSettingsOnUpdate()

	local settings = ReticleSettings.SavedVars

	-- Check states
	local inMouseMode = IsGameCameraUIModeActive()
	local stealthed = GetUnitStealthState( "player" ) ~= STEALTH_STATE_NONE
	local inDisguise = GetUnitDisguiseState( "player" ) ~= DISGUISE_STATE_NONE
	local target = ( GetUnitNameHighlightedByReticle() ~= "" and IsGameCameraUnitHighlightedAttackable() )
	local inCombat = IsUnitInCombat( "player" )
	local interactable = GetGameCameraInteractableInfo()
	if ( IsUnitPlayer( "reticleover" ) ) then
		if ( settings.interactablePlayer and ReticleSettings.GetTargetDistance(1) <= 1.2 ) then
			interactable = true
		end
	end
	
	-- Config Reticle only when not in mouse mode.
	if ( not inMouseMode ) then
	
		-- Variables
		if ( currentReticle == "Nothing" ) then reticleVisible = true else reticleVisible = false end
		
		local interactTextScale = ReticleSettings.SavedVars.textScale / 100
		
		local rcNormal = settings.normalReticle
		local rcInteractable = settings.interactableReticle
		local rcTarget = settings.targetReticle
		local rcCombat = settings.combatReticle
		
		-- Default Reticle
		if ( not inCombat ) then loadCustomReticle( rcNormal ) end
		local reticleScale = ReticleSettings.SavedVars.normalScale / 100
		local reticleAlpha = settings.normalAlpha / 100
		-- Color
		if ( settings.normalReticleReactionColor ) then
			if ( currentReticle ~= "Default" ) then
				RS_ReticleContainer:SetColor( GetUnitReactionColor( "reticleover" ) )
			else
				ZO_ReticleContainerReticle:SetColor( GetUnitReactionColor( "reticleover" ) )
			end
		else
			if ( currentReticle ~= "Default" ) then
				RS_ReticleContainer:SetColor( unpack( ReticleSettings.GetCustomColor( settings.normalReticleColor ) ) )
			else
				ZO_ReticleContainerReticle:SetColor( unpack( ReticleSettings.GetCustomColor( settings.normalReticleColor ) ) )
			end
		end
		
		-- Change settings according to the state of player.
		if ( target ) then
			if ( ReticleSettings.GetTargetDistance(1) <= MeleeRangeDistance ) then
				-- Melee
				loadCustomReticle( settings.targetMeleeReticle )
				if ( currentReticle == "Nothing" ) then reticleVisible = true else reticleVisible = false end
				reticleScale = settings.targetMeleeReticleScale / 100
				reticleAlpha = settings.targetMeleeReticleAlpha / 100
				if ( settings.targetMeleeReticleReactionColor ) then
					if ( currentReticle ~= "Default" ) then
						RS_ReticleContainer:SetColor( GetUnitReactionColor( "reticleover" ) )
					else
						ZO_ReticleContainerReticle:SetColor( GetUnitReactionColor( "reticleover" ) )
					end
				else
					if ( currentReticle ~= "Default" ) then
						RS_ReticleContainer:SetColor( unpack( ReticleSettings.GetCustomColor( settings.targetMeleeReticleColor ) ) )
					else
						ZO_ReticleContainerReticle:SetColor( unpack( ReticleSettings.GetCustomColor( settings.targetMeleeReticleColor ) ) )
					end
				end
			else
				-- Range
				loadCustomReticle( rcTarget )
				if ( currentReticle == "Nothing" ) then reticleVisible = true else reticleVisible = false end
				reticleScale = settings.targetScale / 100
				reticleAlpha = settings.targetAlpha / 100
				if ( settings.targetReticleReactionColor ) then
					if ( currentReticle ~= "Default" ) then
						RS_ReticleContainer:SetColor( GetUnitReactionColor( "reticleover" ) )
					else
						ZO_ReticleContainerReticle:SetColor( GetUnitReactionColor( "reticleover" ) )
					end
				else
					if ( currentReticle ~= "Default" ) then
						RS_ReticleContainer:SetColor( unpack( ReticleSettings.GetCustomColor( settings.targetReticleColor ) ) )
					else
						ZO_ReticleContainerReticle:SetColor( unpack( ReticleSettings.GetCustomColor( settings.targetReticleColor ) ) )
					end
				end
			end
		elseif ( inCombat ) then
			loadCustomReticle( rcCombat )
			if ( currentReticle == "Nothing" ) then reticleVisible = true else reticleVisible = false end
			reticleScale = settings.combatScale / 100
			reticleAlpha = settings.combatAlpha / 100
			-- Color
			if ( settings.combatReticleReactionColor ) then
				if ( currentReticle ~= "Default" ) then
					RS_ReticleContainer:SetColor( GetUnitReactionColor( "reticleover" ) )
				else
					ZO_ReticleContainerReticle:SetColor( GetUnitReactionColor( "reticleover" ) )
				end
			else
				if ( currentReticle ~= "Default" ) then
					RS_ReticleContainer:SetColor( unpack( ReticleSettings.GetCustomColor( settings.combatReticleColor ) ) )
				else
					ZO_ReticleContainerReticle:SetColor( unpack( ReticleSettings.GetCustomColor( settings.combatReticleColor ) ) )
				end
			end
		elseif ( interactable ) then
			loadCustomReticle( rcInteractable )
			if ( currentReticle == "Nothing" ) then reticleVisible = true else reticleVisible = false end
			reticleScale = settings.interactableScale / 100
			reticleAlpha = settings.interactableAlpha / 100
			-- Color
			if ( settings.interactableReticleReactionColor ) then
				if ( currentReticle ~= "Default" ) then
					RS_ReticleContainer:SetColor( GetUnitReactionColor( "reticleover" ) )
				else
					ZO_ReticleContainerReticle:SetColor( GetUnitReactionColor( "reticleover" ) )
				end
			else
				if ( currentReticle ~= "Default" ) then
					RS_ReticleContainer:SetColor( unpack( ReticleSettings.GetCustomColor( settings.interactableReticleColor ) ) )
				else
					ZO_ReticleContainerReticle:SetColor( unpack( ReticleSettings.GetCustomColor( settings.interactableReticleColor ) ) )
				end
			end
		end
		if ( stealthed or inDisguise ) then
			if ( currentReticle == "Nothing" ) then reticleVisible = true else reticleVisible = false end
			reticleScale = settings.stealthedScale / 100
			reticleAlpha = settings.stealthedAlpha / 100
			if ( settings.stealthedReticleReactionColor ) then
				if ( currentReticle ~= "Default" ) then
					RS_ReticleContainer:SetColor( GetUnitReactionColor( "reticleover" ) )
				else
					ZO_ReticleContainerReticle:SetColor( GetUnitReactionColor( "reticleover" ) )
				end
			else
				if ( currentReticle ~= "Default" ) then
					RS_ReticleContainer:SetColor( unpack( ReticleSettings.GetCustomColor( settings.stealthedReticleColor ) ) )
				else
					ZO_ReticleContainerReticle:SetColor( unpack( ReticleSettings.GetCustomColor( settings.stealthedReticleColor ) ) )
				end
			end
		end
		
		-- Adjust the size of Interact.
		ZO_ReticleContainerInteract:SetScale(interactTextScale)
				
		-- Adjust and display Reticle.
		if ( stealthed or inDisguise ) then
			ZO_ReticleContainerStealthEye:SetHidden(reticleVisible)
			ZO_ReticleContainerStealthEye:SetScale(reticleScale)
			ZO_ReticleContainerStealthEye:SetAlpha(reticleAlpha)
			ZO_ReticleContainerStealthText:SetAlpha(reticleAlpha)
			ZO_ReticleContainerStealthText:SetHidden(not settings.stealthedTextVisible)
			RS_ReticleContainer:SetHidden(true)
		else
			if ( currentReticle ~= "Default" ) then
				ZO_ReticleContainerReticle:SetHidden(true)
				RS_ReticleContainer:SetHidden(reticleVisible)
				RS_ReticleContainer:SetScale(reticleScale)
				RS_ReticleContainer:SetAlpha(reticleAlpha)
			else
				RS_ReticleContainer:SetHidden(true)
				ZO_ReticleContainerReticle:SetHidden(reticleVisible)
				ZO_ReticleContainerReticle:SetScale(reticleScale)
				ZO_ReticleContainerReticle:SetAlpha(reticleAlpha)
			end
		end
		
		-- Update labels
		ReticleSettings.updateTopLabel( reticleVisible )
		ReticleSettings.updateRightLabel( reticleVisible )
		ReticleSettings.updateBottomLabel( reticleVisible )
		ReticleSettings.updateLeftLabel( reticleVisible )
	
		-- Show or hide target unit frame.
		ZO_TargetUnitFramereticleover:SetHidden( not settings.optShowUnitFrame )
	
	else
	
		RS_TopLabel:SetHidden(true)
		RS_RightLabel:SetHidden(true)
		RS_BottomLabel:SetHidden(true)
		RS_LeftLabel:SetHidden(true)
	
	end
	
end

local function GetFontString( font )
-- code by EnSabahNur

	local format = "%s|%d"

	if font.style ~= "none" then
		format = format .. "|%s"
	end

	return format:format( LMP:Fetch( LMP.MediaType.FONT, ReticleSettings.SavedVars.Font ), 16, 'soft-shadow-thin' )
	
end

-- Use this function to update Top Label module.
function ReticleSettings.updateTopLabel( reticleVisible )

	local settings = ReticleSettings.SavedVars

	local option = settings.topLabelValue
	local linkReticle = settings.topLabelLinkReticle
	local fontString = GetFontString(settings.Font)
	
	if ( linkReticle ) then
		if ( reticleVisible ) then option = "Nothing" end
	end
	
	if ( option ~= "Nothing" ) then
	
		RS_TopLabel:SetHidden(false)
	
		-- Set text
		RS_TopLabel:SetText( ReticleSettings.GetLabelText( option ) )
	
		-- Set font
		RS_TopLabel:SetFont(fontString)
	
		-- Set scale
		RS_TopLabel:SetScale( settings.topLabelScale / 100 )
		
		-- Set position
		RS_TopLabel:ClearAnchors()
		RS_TopLabel:SetAnchor( CENTER, ZO_ReticleContainer, CENTER, 0, settings.topLabelPos )

		-- Set color and transparency
		local a = settings.topLabelAlpha / 100
		if ( settings.topLabelShowColor and DoesUnitExist( "reticleover" ) ) then
			local r, g, b = GetUnitReactionColor( "reticleover" )
			RS_TopLabel:SetColor( r, g, b, a )
		else
			local r, g, b = unpack( ReticleSettings.GetCustomColor( settings.topLabelCustomColor ) )
			RS_TopLabel:SetColor( r, g, b, a )
		end
		
	else
	
		RS_TopLabel:SetHidden(true)
		
	end

end

-- Use this function to update Right Label module.
function ReticleSettings.updateRightLabel( reticleVisible )

	local settings = ReticleSettings.SavedVars

	local option = settings.rightLabelValue
	local linkReticle = settings.rightLabelLinkReticle
	local fontString = GetFontString(settings.Font)
	
	if ( linkReticle ) then
		if ( reticleVisible ) then option = "Nothing" end
	end
	
	if ( option ~= "Nothing" ) then
	
		RS_RightLabel:SetHidden(false)
		
		-- Set text
		RS_RightLabel:SetText( ReticleSettings.GetLabelText( option ) )
	
		-- Set font
		RS_RightLabel:SetFont(fontString)
	
		-- Set scale
		RS_RightLabel:SetScale( settings.rightLabelScale / 100 )
		
		-- Set position
		RS_RightLabel:ClearAnchors()
		RS_RightLabel:SetAnchor( CENTER, ZO_ReticleContainer, CENTER, settings.rightLabelPos, 0 )

		-- Set color and transparency
		local a = settings.rightLabelAlpha / 100
		if ( settings.rightLabelShowColor ) then
			local r, g, b = GetUnitReactionColor( "reticleover" )
			RS_RightLabel:SetColor( r, g, b, a )
		else
			local r, g, b = unpack( ReticleSettings.GetCustomColor( settings.rightLabelCustomColor ) )
			RS_RightLabel:SetColor( r, g, b, a )
		end
		
	else
	
		RS_RightLabel:SetHidden(true)
		
	end

end

-- Use this function to update Bottom Label module.
function ReticleSettings.updateBottomLabel( reticleVisible )

	local settings = ReticleSettings.SavedVars

	local option = settings.bottomLabelValue
	local linkReticle = settings.bottomLabelLinkReticle
	local fontString = GetFontString(settings.Font)
		
	if ( linkReticle ) then
		if ( reticleVisible ) then option = "Nothing" end
	end
		
	if ( option ~= "Nothing" ) then
		
		RS_BottomLabel:SetHidden(false)
	
		-- Set text
		RS_BottomLabel:SetText( ReticleSettings.GetLabelText( option ) )
	
		-- Set font
		RS_BottomLabel:SetFont(fontString)
	
		-- Set scale
		RS_BottomLabel:SetScale( settings.bottomLabelScale / 100 )
	
		-- Set position
		RS_BottomLabel:ClearAnchors()
		RS_BottomLabel:SetAnchor( CENTER, ZO_ReticleContainer, CENTER, 0, settings.bottomLabelPos )

		-- Set color and transparency
		local a = settings.bottomLabelAlpha / 100
		if ( settings.bottomLabelShowColor and DoesUnitExist( "reticleover" ) ) then
			local r, g, b = GetUnitReactionColor( "reticleover" )
			RS_BottomLabel:SetColor( r, g, b, a )
		else
			local r, g, b = unpack( ReticleSettings.GetCustomColor( settings.bottomLabelCustomColor ) )
			RS_BottomLabel:SetColor( r, g, b, a )
		end
		
	else
	
		RS_BottomLabel:SetHidden(true)
		
	end
	
end

-- Use this function to update Left Label module.
function ReticleSettings.updateLeftLabel( reticleVisible )

	local settings = ReticleSettings.SavedVars

	local option = settings.leftLabelValue
	local linkReticle = settings.leftLabelLinkReticle
	local fontString = GetFontString(settings.Font)
	
	if ( linkReticle ) then
		if ( reticleVisible ) then option = "Nothing" end
	end
	
	if ( option ~= "Nothing" ) then
	
		RS_LeftLabel:SetHidden(false)
	
		-- Set text
		RS_LeftLabel:SetText( ReticleSettings.GetLabelText( option ) )
	
		-- Set font
		RS_LeftLabel:SetFont(fontString)
	
		-- Set scale
		RS_LeftLabel:SetScale( settings.leftLabelScale / 100 )
	
		-- Set position
		RS_LeftLabel:ClearAnchors()
		RS_LeftLabel:SetAnchor( CENTER, ZO_ReticleContainer, CENTER, settings.leftLabelPos, 0 )

		-- Set color and transparency
		local a = settings.leftLabelAlpha / 100
		if ( settings.leftLabelShowColor and DoesUnitExist( "reticleover" ) ) then
			local r, g, b = GetUnitReactionColor( "reticleover" )
			RS_LeftLabel:SetColor( r, g, b, a )
		else
			local r, g, b = unpack( ReticleSettings.GetCustomColor( settings.leftLabelCustomColor ) )
			RS_LeftLabel:SetColor( r, g, b, a )
		end
		
	else
	
		RS_LeftLabel:SetHidden(true)
		
	end

end

-- This function will return the requested string for Labels.
-- Such as target's name, level, health, caption, etc.
function ReticleSettings.GetLabelText( option )

	-- Variables
	local labelText = ""
	
	-- Get info
	local targetName = GetUnitName( "reticleover" )
	local targetReaction = GetUnitReaction( "reticleover" )
	
	---- Clean up the format of data
	-- Target Name
	if ( targetName == nil ) then targetName = "" end
	
	---- Options
	-- Nothing
	if ( option == "Nothing" ) then
		labelText = ""
	-- Target Name
	elseif ( option == "Target Name" ) then
		labelText = targetName
	-- Target Name + Caption
	elseif ( option == "Target Name + Caption" ) then
		labelText = targetName
		if ( targetName ~= "" ) then
			if ( not IsUnitPlayer( "reticleover" ) ) then
				if ( targetReaction == 5 or targetReaction == 7 ) then
					if ( targetCaption ~= "" ) then
						labelText = labelText .. "\n" .. GetTargetCaption()
					end
				end
			end
		end
	-- Target Name + Level
	elseif ( option == "Target Name + Level" ) then
		labelText = targetName .. "  " .. ReticleSettings.GetTargetLevel()
	-- Target Name + Level + Caption
	elseif ( option == "Target Name + Level + Caption" ) then
		labelText = targetName .. "  " .. ReticleSettings.GetTargetLevel()
		if ( targetName ~= "" ) then
			if ( not IsUnitPlayer( "reticleover" ) ) then
				if ( targetReaction == 5 or targetReaction == 7 ) then
					if ( targetCaption ~= "" ) then
						labelText = labelText .. "\n" .. GetTargetCaption()
					end
				end
			end
		end
	-- Target Name + Level + HP
	elseif ( option == "Target Name + Level + HP" ) then
		labelText = targetName .. "  " .. ReticleSettings.GetTargetLevel()
		local targetHealthCurrent, targetHealthMax = GetUnitPower( "reticleover", POWERTYPE_HEALTH )
		local targetHealthPercent = math.floor( targetHealthCurrent / targetHealthMax * 100 )
		if ( targetHealthMax ~= 0 ) then
			labelText = labelText .. "\n" .. "HP: " .. targetHealthCurrent .. " / " .. targetHealthMax .. " (" .. targetHealthPercent .. "%)"
		end
	-- Target HP (Min/Max)
	elseif ( option == "Target HP (Current/Max)" ) then
		local targetHealthCurrent, targetHealthMax = GetUnitPower( "reticleover", POWERTYPE_HEALTH )
		if ( targetHealthMax ~= 0 ) then
			labelText = "HP: " .. targetHealthCurrent .. " / " .. targetHealthMax
		end
	-- Target HP (Percent)
	elseif ( option == "Target HP (Percent)" ) then
		local targetHealthCurrent, targetHealthMax = GetUnitPower( "reticleover", POWERTYPE_HEALTH )
		local targetHealthPercent = math.floor( targetHealthCurrent / targetHealthMax * 100 )
		if ( targetHealthMax ~= 0 ) then
			labelText = targetHealthPercent .. "%"
		end
	-- Target HP (Min/Max/Percent)
	elseif ( option == "Target HP (Current/Max/Percent)" ) then
		local targetHealthCurrent, targetHealthMax = GetUnitPower( "reticleover", POWERTYPE_HEALTH )
		local targetHealthPercent = math.floor( targetHealthCurrent / targetHealthMax * 100 )
		if ( targetHealthMax ~= 0 ) then
			labelText = "HP: " .. targetHealthCurrent .. " / " .. targetHealthMax .. " (" .. targetHealthPercent .. "%)"
		end
	-- Target Distance
	elseif ( option == "Target Distance" ) then
		if ( targetName ~= "" ) then
			labelText = ReticleSettings.GetTargetDistance(0) .. "m"
		end
	-- Target Caption
	elseif ( option == "Target Caption" ) then
		if ( targetName ~= "" ) then
			if ( not IsUnitPlayer( "reticleover" ) ) then
				if ( targetReaction == 5 or targetReaction == 7 ) then
					labelText = GetTargetCaption()
				end
			end
		end
	-- Active Weapon
	elseif ( option == "Active Weapon" ) then
		--if ( ActiveWeapon == 0 or 1 ) then
		if ( ActiveWeapon == 0 or ACTIVE_WEAPON_PAIR_MAIN ) then
			labelText = "Primary"
		elseif ( ActiveWeapon == ACTIVE_WEAPON_PAIR_BACKUP ) then
			labelText = "Backup"
		else
			labelText = ""
		end
	end

	return labelText
	
end

-- Return target level or veteran rank.
function ReticleSettings.GetTargetLevel()

	local value = ""
	local targetLevel = GetUnitLevel( "reticleover" )
	local targetVeteranRank = GetUnitVeteranRank( "reticleover" )

	if ( targetVeteranRank > 0 and targetVeteranRank ~= 0 ) then
		value = "[ VR" .. targetVeteranRank .. " ]"
	elseif ( targetLevel > 0 ) then
		value = "[ " .. targetLevel .. " ]"
	end
	
	return value
	
end

function GetTargetCaption()
-- code by EnSabahNur
	
	local targetCaption = GetUnitCaption( "reticleover" )
	
	if ( targetCaption == nil ) then
		targetCaption = ""
	else
		local pos = string.find ( targetCaption, "^", 1, true ) --ESN
		if ( pos ) then
			targetCaption = string.sub ( targetCaption, 1, pos - 1 )
		end
	end
	
	return targetCaption

end

-- Return the distance value between target and player.
function ReticleSettings.GetTargetDistance( decimal )
-- Code from Azurah by Kith

	local GetMapNumTiles 			= GetMapNumTiles
	local GetMapType 				= GetMapType
	local GetMapContentType 		= GetMapContentType
	local GetMapPlayerPosition 		= GetMapPlayerPosition
	local math_sqrt					= math.sqrt
	local math_pow					= math.pow
	local math_max					= math.max
	local strformat					= string.format

	local mapWidth, mapHeight, mapType, mapContent
	local pX, pY, pZ, tX, tY, tZ
	local range, div
	
	mapWidth, mapHeight = GetMapNumTiles()
	mapType = GetMapType()
	mapContent = GetMapContentType()
	pX, pY, pZ = GetMapPlayerPosition( 'player' )
	tX, tY, tZ = GetMapPlayerPosition( 'reticleover' )
	
	if ( mapContent == 0 ) then
		div = mapType == 1 and 15 or 2.50
	elseif ( mapContent == 1 ) then
		div = mapType == 1 and 25 or 100
	elseif ( mapContent == 2 ) then
		div = mapType == 1 and 21 or 26
	else
		div = mapType * mapWidth
	end
	
	local distanceValue = math_max( 0, ( math_sqrt( math_pow( ( pX / div ) - ( tX / div ), 2 ) + math_pow( ( pY / div ) - ( tY / div ), 2 ) ) * 10000 ) - 1 ) / 3
	distanceValue = ReticleSettings.round( distanceValue, decimal )
	if ( distanceValue <= 0 ) then distanceValue = 0 end
	
	return distanceValue
	
end

-- This function load up first thing to setup whatever needed to get this Addon to run properly,
-- and create settings panel for in-game options menu.
function ReticleSettings.OnInitialized(eventCode, addOnName)
	
	-- Only initialize this addon.
	if ( addOnName ~= "ReticleSettings" ) then return end
	
	-- Setup the default values.
	ReticleSettings.Defaults = {
		-- Options
		["optShowUnitFrame"] = true,
		["optInteractTextAltPosition"] = false,
		["textScale"] = 100,
		["Font"] = "Univers 67",
		-- Reticle: Normal
		["normalReticle"] = "Dot",
		["normalReticleColor"] = "ffffff",
		["normalReticleReactionColor"] = true,
		["normalScale"] = 20,
		["normalAlpha"] = 100,
		-- Reticle: Interactable
		["interactableReticle"] = "Default",
		["interactableReticleColor"] = "ffffff",
		["interactableReticleReactionColor"] = true,
		["interactablePlayer"] = true,
		["interactableScale"] = 80,
		["interactableAlpha"] = 100,
		-- Reticle: Stealthed or Disguise
		["stealthedReticleColor"] = "ffffff",
		["stealthedReticleReactionColor"] = true,
		["stealthedVisible"] = true,
		["stealthedTextVisible"] = true,
		["stealthedScale"] = 70,
		["stealthedAlpha"] = 100,
		-- Reticle: Target In-Range (Melee)
		["targetMeleeReticle"] = "Crosshair",
		["targetMeleeReticleColor"] = "ffffff",
		["targetMeleeReticleReactionColor"] = true,
		["targetMeleeReticleScale"] = 50,
		["targetMeleeReticleAlpha"] = 100,
		-- Reticle: Target In-Range (Range)
		["targetReticle"] = "Crosshair",
		["targetReticleColor"] = "ffffff",
		["targetReticleReactionColor"] = true,
		["targetScale"] = 50,
		["targetAlpha"] = 100,
		-- Reticle: In Combat
		["combatReticle"] = "Round",
		["combatReticleColor"] = "ffffff",
		["combatReticleReactionColor"] = true,
		["combatScale"] = 60,
		["combatAlpha"] = 80,
		-- Top Label
		["topLabelValue"] = "Nothing",
		["topLabelShowColor"] = true,
		["topLabelLinkReticle"] = false,
		["topLabelCustomColor"] = "ffffff",
		["topLabelScale"] = 100,
		["topLabelAlpha"] = 70,
		["topLabelPos"] = -150,
		-- Right Label
		["rightLabelValue"] = "Nothing",
		["rightLabelShowColor"] = true,
		["rightLabelLinkReticle"] = false,
		["rightLabelCustomColor"] = "ffffff",
		["rightLabelScale"] = 100,
		["rightLabelAlpha"] = 70,
		["rightLabelPos"] = 350,
		-- Bottom Label
		["bottomLabelValue"] = "Nothing",
		["bottomLabelShowColor"] = true,
		["bottomLabelLinkReticle"] = false,
		["bottomLabelCustomColor"] = "ffffff",
		["bottomLabelScale"] = 100,
		["bottomLabelAlpha"] = 70,
		["bottomLabelPos"] = 150,
		-- Left Label
		["leftLabelValue"] = "Nothing",
		["leftLabelShowColor"] = true,
		["leftLabelLinkReticle"] = false,
		["leftLabelCustomColor"] = "ffffff",
		["leftLabelScale"] = 100,
		["leftLabelAlpha"] = 70,
		["leftLabelPos"] = -350
	}
	
	-- Create or load saved variables.
	ReticleSettings.SavedVars = ZO_SavedVars:NewAccountWide("ReticleSettings_SavedVariables", 1, nil, ReticleSettings.Defaults)
	
	-- Create Settings
	local panel = LAM:CreateControlPanel( "_reticleSettings", "Reticle")
	
	-- OPTIONS
	LAM:AddHeader( panel, "headerOptions", "Options" )
	-- Show Unit Frame [ON/OFF]
	LAM:AddCheckbox( panel, "optShowUnitFrame", "Unit Frame", "Turn off to hide unit frame (the one under the compass that show up when you target player or NPC).",
		function() return ReticleSettings.SavedVars.optShowUnitFrame end,
		function() ReticleSettings.SavedVars.optShowUnitFrame = not ReticleSettings.SavedVars.optShowUnitFrame end )
	-- Interact Text Alternative Position [ON/OFF]
	LAM:AddCheckbox( panel, "optInteractTextAltPosition", "Interact Text Alternative Position", "Move Intereact Text to alternative position, away from the labels around the Reticle.",
		function() return ReticleSettings.SavedVars.optInteractTextAltPosition end,
		function()
			ReticleSettings.SavedVars.optInteractTextAltPosition = not ReticleSettings.SavedVars.optInteractTextAltPosition
			ReloadUI()
		end, true, "Reloads UI" )
	-- Interact Text Scale
	LAM:AddSlider( panel, "textScale", "Interact Text Scale", "Adjust the size of interact text.", 50, 200, 1,
		function() return ReticleSettings.SavedVars.textScale end,
		function( scale ) ReticleSettings.SavedVars.textScale = scale end )
	-- Label Font
	LAM:AddDropdown(panel,	"Font", "Label Font", "Pick something!", LMP:List('font'),
		function() return ReticleSettings.SavedVars.Font end,
		function( font ) ReticleSettings.SavedVars.Font = font end )
	
	-- RETICLE: NORMAL
	LAM:AddHeader( panel, "headerNormal", "Reticle: Normal" )
	-- Reticle Style
	LAM:AddDropdown( panel, "normalReticle", "Reticle Style", "Choose which Reticle to display as default.",
		RETICLE_OPTIONS,
		function() return ReticleSettings.SavedVars.normalReticle end,
		function( value ) ReticleSettings.SavedVars.normalReticle = value end )
	-- Color
	LAM:AddColorPicker( panel, "normalReticleColor", "Color", "Choose color for Normal Reticle.",
		function() return ReticleSettings.HexToRGB( ReticleSettings.SavedVars.normalReticleColor ) end,
		function( r, g, b ) ReticleSettings.SavedVars.normalReticleColor = ReticleSettings.RGBToHex( r, g, b ) end )
	--  Use Reaction Color
	LAM:AddCheckbox( panel, "normalReticleReactionColor", "Use Reaction Color", "Change color of Reticle according to the type of target.",
		function() return ReticleSettings.SavedVars.normalReticleReactionColor end,
		function() ReticleSettings.SavedVars.normalReticleReactionColor = not ReticleSettings.SavedVars.normalReticleReactionColor end )
	-- Scale
	LAM:AddSlider( panel, "normalScale", "Scale", "", 15, 100, 5,
		function() return ReticleSettings.SavedVars.normalScale end,
		function( scale ) ReticleSettings.SavedVars.normalScale = scale end )
	-- Transparency
	LAM:AddSlider( panel, "normalAlpha", "Transparency", "", 30, 100, 1,
		function() return ReticleSettings.SavedVars.normalAlpha end,
		function ( alpha ) ReticleSettings.SavedVars.normalAlpha = alpha end )
	
	-- RETICLE: INTERACTABLE
	LAM:AddHeader( panel, "headerInteractable", "Reticle: Interactable" )
	-- Reticle Styel
	LAM:AddDropdown( panel, "interactableReticle", "Reticle Style", "Choose which Reticle to display when target is interactable.",
		RETICLE_OPTIONS,
		function() return ReticleSettings.SavedVars.interactableReticle end,
		function( value ) ReticleSettings.SavedVars.interactableReticle = value end )
	-- Color
	LAM:AddColorPicker( panel, "interactableReticleColor", "Color", "Choose color for Interact Reticle.",
		function() return ReticleSettings.HexToRGB( ReticleSettings.SavedVars.interactableReticleColor ) end,
		function( r, g, b ) ReticleSettings.SavedVars.interactableReticleColor = ReticleSettings.RGBToHex( r, g, b ) end )
	-- Use Reaction Color
	LAM:AddCheckbox( panel, "interactableReticleReactionColor", "Use Reaction Color", "Change color of Reticle according to the type of target.",
		function() return ReticleSettings.SavedVars.interactableReticleReactionColor end,
		function() ReticleSettings.SavedVars.interactableReticleReactionColor = not ReticleSettings.SavedVars.interactableReticleReactionColor end )
	-- Interactable Player [ON/OFF]
	LAM:AddCheckbox( panel, "interactablePlayer", "Interactable Player", "Show Interact Reticle on target player.",
		function() return ReticleSettings.SavedVars.interactablePlayer end,
		function() ReticleSettings.SavedVars.interactablePlayer = not ReticleSettings.SavedVars.interactablePlayer end )
	-- Scale
	LAM:AddSlider( panel, "interactableScale", "Scale", "", 15, 100, 5,
		function() return ReticleSettings.SavedVars.interactableScale end,
		function( scale ) ReticleSettings.SavedVars.interactableScale = scale end )
	-- Transparency
	LAM:AddSlider( panel, "interactableAlpha", "Transparency", "", 30, 100, 1,
		function() return ReticleSettings.SavedVars.interactableAlpha end,
		function ( alpha ) ReticleSettings.SavedVars.interactableAlpha = alpha end )
	
	-- RETICLE: STEALTHED
	LAM:AddHeader( panel, "headerStealthed", "Reticle: Stealthed" )
	-- Color
	LAM:AddColorPicker( panel, "stealthedReticleColor", "Color", "Choose color for Stealted Reticle.",
		function() return ReticleSettings.HexToRGB( ReticleSettings.SavedVars.stealthedReticleColor ) end,
		function( r, g, b ) ReticleSettings.SavedVars.stealthedReticleColor = ReticleSettings.RGBToHex( r, g, b ) end )
	-- Use Reaction Color
	LAM:AddCheckbox( panel, "stealthedReticleReactionColor", "Use Reaction Color", "Change color of Reticle according to the type of target.",
		function() return ReticleSettings.SavedVars.stealthedReticleReactionColor end,
		function() ReticleSettings.SavedVars.stealthedReticleReactionColor = not ReticleSettings.SavedVars.stealthedReticleReactionColor end )
	-- Show Reticle [ON/OFF]
	LAM:AddCheckbox( panel, "stealthedVisible", "Show Reticle", "Show or hide stealthed and disguise Reticle",
		function() return ReticleSettings.SavedVars.stealthedVisible end,
		function() ReticleSettings.SavedVars.stealthedVisible = not ReticleSettings.SavedVars.stealthedVisible end )
	-- Status Text [ON/OFF]
	LAM:AddCheckbox( panel, "stealthedTextVisible", "Status Text", "Show or hide HIDDEN and DISGUISE text.",
		function() return ReticleSettings.SavedVars.stealthedTextVisible end,
		function() ReticleSettings.SavedVars.stealthedTextVisible = not ReticleSettings.SavedVars.stealthedTextVisible end )
	-- Scale
	LAM:AddSlider( panel, "stealthedScale", "Scale", "", 50, 100, 5,
		function() return ReticleSettings.SavedVars.stealthedScale end,
		function( scale ) ReticleSettings.SavedVars.stealthedScale = scale end )
	-- Transparency
	LAM:AddSlider( panel, "stealthedAlpha", "Transparency", "", 30, 100, 1,
		function() return ReticleSettings.SavedVars.stealthedAlpha end,
		function ( alpha ) ReticleSettings.SavedVars.stealthedAlpha = alpha end )
	
	-- RETICLE: TARGET IN-RANGE (MELEE)
	LAM:AddHeader( panel, "headerTargetMelee", "Reticle: Target In-Range (Melee)" )
	-- Reticle Style
	LAM:AddDropdown( panel, "targetMeleeReticle", "Reticle Style", "Choose which Reticle to display when target is in attackable melee range.",
		RETICLE_OPTIONS,
		function() return ReticleSettings.SavedVars.targetMeleeReticle end,
		function( value ) ReticleSettings.SavedVars.targetMeleeReticle = value end )
	-- Color
	LAM:AddColorPicker( panel, "targetMeleeReticleColor", "Color", "Choose color for Target In-Range (Melee) Reticle.",
		function() return ReticleSettings.HexToRGB( ReticleSettings.SavedVars.targetMeleeReticleColor ) end,
		function( r, g, b ) ReticleSettings.SavedVars.targetMeleeReticleColor = ReticleSettings.RGBToHex( r, g, b ) end )
	-- Use Reaction Color
	LAM:AddCheckbox( panel, "targetMeleeReticleReactionColor", "Use Reaction Color", "Change color of Reticle according to the type of target.",
		function() return ReticleSettings.SavedVars.targetMeleeReticleReactionColor end,
		function() ReticleSettings.SavedVars.targetMeleeReticleReactionColor = not ReticleSettings.SavedVars.targetMeleeReticleReactionColor end )
	-- Scale
	LAM:AddSlider( panel, "targetMeleeReticleScale", "Scale", "", 15, 100, 5,
		function() return ReticleSettings.SavedVars.targetMeleeReticleScale end,
		function( scale ) ReticleSettings.SavedVars.targetMeleeReticleScale = scale end )
	-- Transparency
	LAM:AddSlider( panel, "targetMeleeReticleAlpha", "Transparency", "", 30, 100, 1,
		function() return ReticleSettings.SavedVars.targetMeleeReticleAlpha end,
		function ( alpha ) ReticleSettings.SavedVars.targetMeleeReticleAlpha = alpha end )
	
	-- RETICLE: TARGET IN-RANGE (RANGE)
	LAM:AddHeader( panel, "headerTarget", "Reticle: Target In-Range (Range)" )
	-- Reticle Style
	LAM:AddDropdown( panel, "targetReticle", "Reticle Style", "Choose which Reticle to display when target is in attackable range.",
		RETICLE_OPTIONS,
		function() return ReticleSettings.SavedVars.targetReticle end,
		function( value ) ReticleSettings.SavedVars.targetReticle = value end )
	-- Color
	LAM:AddColorPicker( panel, "targetReticleColor", "Color", "Choose color for Target In-Range (Range) Reticle.",
		function() return ReticleSettings.HexToRGB( ReticleSettings.SavedVars.targetReticleColor ) end,
		function( r, g, b ) ReticleSettings.SavedVars.targetReticleColor = ReticleSettings.RGBToHex( r, g, b ) end )
	-- Use Reaction Color
	LAM:AddCheckbox( panel, "targetReticleReactionColor", "Use Reaction Color", "Change color of Reticle according to the type of target.",
		function() return ReticleSettings.SavedVars.targetReticleReactionColor end,
		function() ReticleSettings.SavedVars.targetReticleReactionColor = not ReticleSettings.SavedVars.targetReticleReactionColor end )
	-- Scale
	LAM:AddSlider( panel, "targetScale", "Scale", "", 15, 100, 5,
		function() return ReticleSettings.SavedVars.targetScale end,
		function( scale ) ReticleSettings.SavedVars.targetScale = scale end )
	-- Transparency
	LAM:AddSlider( panel, "targetAlpha", "Transparency", "", 30, 100, 1,
		function() return ReticleSettings.SavedVars.targetAlpha end,
		function ( alpha ) ReticleSettings.SavedVars.targetAlpha = alpha end )
	
	-- RETICLE: IN COMBAT
	LAM:AddHeader( panel, "headerCombat", "Reticle: In Combat" )
	-- Reticle Style
	LAM:AddDropdown( panel, "combatReticle", "Reticle Style", "Choose which Reticle to display when in combat.",
		RETICLE_OPTIONS,
		function() return ReticleSettings.SavedVars.combatReticle end,
		function( value ) ReticleSettings.SavedVars.combatReticle = value end )
	-- Color
	LAM:AddColorPicker( panel, "combatReticleColor", "Color", "Choose color for In-Combat Reticle.",
		function() return ReticleSettings.HexToRGB( ReticleSettings.SavedVars.combatReticleColor ) end,
		function( r, g, b ) ReticleSettings.SavedVars.combatReticleColor = ReticleSettings.RGBToHex( r, g, b ) end )
	-- Use Reaction Color
	LAM:AddCheckbox( panel, "combatReticleReactionColor", "Use Reaction Color", "Change color of Reticle according to the type of target.",
		function() return ReticleSettings.SavedVars.combatReticleReactionColor end,
		function() ReticleSettings.SavedVars.combatReticleReactionColor = not ReticleSettings.SavedVars.combatReticleReactionColor end )
	-- Scale
	LAM:AddSlider( panel, "combatScale", "Scale", "", 15, 100, 5,
		function() return ReticleSettings.SavedVars.combatScale end,
		function( scale ) ReticleSettings.SavedVars.combatScale = scale end )
	-- Transparency
	LAM:AddSlider( panel, "combatAlpha", "Transparency", "", 30, 100, 1,
		function() return ReticleSettings.SavedVars.combatAlpha end,
		function ( alpha ) ReticleSettings.SavedVars.combatAlpha = alpha end )
	
	-- TOP LABEL
	LAM:AddHeader( panel, "topLabel", "Label: Top" )
	-- Show
	LAM:AddDropdown( panel, "topLabelOptions", "Show", "Choose which information to display.", LABEL_OPTIONS,
		function() return ReticleSettings.SavedVars.topLabelValue end,
		function( value ) ReticleSettings.SavedVars.topLabelValue = value end )
	-- Color
	LAM:AddColorPicker( panel, "topLabelCustomColor", "Color", "Choose label color.",
		function() return ReticleSettings.HexToRGB( ReticleSettings.SavedVars.topLabelCustomColor ) end,
		function( r, g, b ) ReticleSettings.SavedVars.topLabelCustomColor = RGBToHex( r, g, b ) end )
	-- Use Reaction Color
	LAM:AddCheckbox( panel, "topLabelColor", "Use Reaction Color", "Change color of top label according to the type of target instead of set color.",
		function() return ReticleSettings.SavedVars.topLabelShowColor end,
		function() ReticleSettings.SavedVars.topLabelShowColor = not ReticleSettings.SavedVars.topLabelShowColor end )
	-- Use Reticle Visibility
	LAM:AddCheckbox( panel, "topLabelLinkReticle", "Use Reticle Visibility", "Hide this label when Reticle is hidden.",
		function() return ReticleSettings.SavedVars.topLabelLinkReticle end,
		function() ReticleSettings.SavedVars.topLabelLinkReticle = not ReticleSettings.SavedVars.topLabelLinkReticle end )
	-- Scale
	LAM:AddSlider( panel, "topLabelScale", "Scale", "", 70, 140, 1,
		function() return ReticleSettings.SavedVars.topLabelScale end,
		function( scale ) ReticleSettings.SavedVars.topLabelScale = scale end )
	-- Transparency
	LAM:AddSlider( panel, "topLabelAlpha", "Transparency", "", 20, 100, 1,
		function() return ReticleSettings.SavedVars.topLabelAlpha end,
		function( scale ) ReticleSettings.SavedVars.topLabelAlpha = scale end )
	-- Position
	LAM:AddSlider( panel, "topLabelPos", "Position", "", -100, -250, -10,
		function() return ReticleSettings.SavedVars.topLabelPos end,
		function( scale ) ReticleSettings.SavedVars.topLabelPos = scale end )

	-- RIGHT LABEL
	LAM:AddHeader( panel, "rightLabel", "Label: Right" )
	-- Show
	LAM:AddDropdown( panel, "rightLabelOptions", "Show", "Choose which information to display.", LABEL_OPTIONS,
		function() return ReticleSettings.SavedVars.rightLabelValue end,
		function( value ) ReticleSettings.SavedVars.rightLabelValue = value end )
	-- Color
	LAM:AddColorPicker( panel, "rightLabelCustomColor", "Color", "Choose label color.",
		function() return ReticleSettings.HexToRGB( ReticleSettings.SavedVars.rightLabelCustomColor ) end,
		function( r, g, b ) ReticleSettings.SavedVars.rightLabelCustomColor = ReticleSettings.RGBToHex( r, g, b ) end )
	-- Use Reaction Color
	LAM:AddCheckbox( panel, "rightLabelColor", "Use Reaction Color", "Change color of right label according to the type of target instead of set color.",
		function() return ReticleSettings.SavedVars.rightLabelShowColor end,
		function() ReticleSettings.SavedVars.rightLabelShowColor = not ReticleSettings.SavedVars.rightLabelShowColor end )
	-- Use Reticle Visibility
	LAM:AddCheckbox( panel, "rightLabelLinkReticle", "Use Reticle Visibility", "Hide this label when Reticle is hidden.",
		function() return ReticleSettings.SavedVars.rightLabelLinkReticle end,
		function() ReticleSettings.SavedVars.rightLabelLinkReticle = not ReticleSettings.SavedVars.rightLabelLinkReticle end )
	-- Scale
	LAM:AddSlider( panel, "rightLabelScale", "Scale", "", 70, 140, 1,
		function() return ReticleSettings.SavedVars.rightLabelScale end,
		function( scale ) ReticleSettings.SavedVars.rightLabelScale = scale end )
	-- Transparency
	LAM:AddSlider( panel, "rightLabelAlpha", "Transparency", "", 20, 100, 1,
		function() return ReticleSettings.SavedVars.rightLabelAlpha end,
		function( scale ) ReticleSettings.SavedVars.rightLabelAlpha = scale end )
	-- Position
	LAM:AddSlider( panel, "rightLabelPos", "Position", "", 300, 500, 10,
		function() return ReticleSettings.SavedVars.rightLabelPos end,
		function( scale ) ReticleSettings.SavedVars.rightLabelPos = scale end )
	
	-- BOTTOM LABEL
	LAM:AddHeader( panel, "bottomLabel", "Label: Bottom" )
	-- Show
	LAM:AddDropdown( panel, "bottomLabelOptions", "Show", "Choose which information to display.", LABEL_OPTIONS,
		function() return ReticleSettings.SavedVars.bottomLabelValue end,
		function( value ) ReticleSettings.SavedVars.bottomLabelValue = value end )
	-- Color
	LAM:AddColorPicker( panel, "bottomLabelCustomColor", "Color", "Choose label color.",
		function() return ReticleSettings.HexToRGB( ReticleSettings.SavedVars.bottomLabelCustomColor ) end,
		function( r, g, b ) ReticleSettings.SavedVars.bottomLabelCustomColor = ReticleSettings.RGBToHex( r, g, b ) end )
	-- Use Reaction Color
	LAM:AddCheckbox( panel, "bottomLabelColor", "Use Reaction Color", "Change color of bottom label according to the type of target instead of set color.",
		function() return ReticleSettings.SavedVars.bottomLabelShowColor end,
		function() ReticleSettings.SavedVars.bottomLabelShowColor = not ReticleSettings.SavedVars.bottomLabelShowColor end )
	-- Use Reticle Visibility
	LAM:AddCheckbox( panel, "bottomLabelLinkReticle", "Use Reticle Visibility", "Hide this label when Reticle is hidden.",
		function() return ReticleSettings.SavedVars.bottomLabelLinkReticle end,
		function() ReticleSettings.SavedVars.bottomLabelLinkReticle = not ReticleSettings.SavedVars.bottomLabelLinkReticle end )
	-- Scale
	LAM:AddSlider( panel, "bottomLabelScale", "Scale", "", 70, 140, 1,
		function() return ReticleSettings.SavedVars.bottomLabelScale end,
		function( scale ) ReticleSettings.SavedVars.bottomLabelScale = scale end )
	-- Transparency
	LAM:AddSlider( panel, "bottomLabelAlpha", "Transparency", "", 20, 100, 1,
		function() return ReticleSettings.SavedVars.bottomLabelAlpha end,
		function( scale ) ReticleSettings.SavedVars.bottomLabelAlpha = scale end )
	-- Position
	LAM:AddSlider( panel, "bottomLabelPos", "Position", "", 100, 250, 10,
		function() return ReticleSettings.SavedVars.bottomLabelPos end,
		function( scale ) ReticleSettings.SavedVars.bottomLabelPos = scale end )
	
	-- LEFT LABEL
	LAM:AddHeader( panel, "leftLabel", "Label: Left" )
	-- Show
	LAM:AddDropdown( panel, "leftLabelOptions", "Show", "", LABEL_OPTIONS,
		function() return ReticleSettings.SavedVars.leftLabelValue end,
		function( value ) ReticleSettings.SavedVars.leftLabelValue = value end )
	-- Color
	LAM:AddColorPicker( panel, "leftLabelCustomColor", "Color", "Choose which information to display.",
		function() return ReticleSettings.HexToRGB( ReticleSettings.SavedVars.leftLabelCustomColor ) end,
		function( r, g, b ) ReticleSettings.SavedVars.leftLabelCustomColor = ReticleSettings.RGBToHex( r, g, b ) end )
	-- Use Reaction Color [ON/OFF]
	LAM:AddCheckbox( panel, "leftLabelColor", "Use Reaction Color", "Change color of left label according to the type of target instead of set color.",
		function() return ReticleSettings.SavedVars.leftLabelShowColor end,
		function() ReticleSettings.SavedVars.leftLabelShowColor = not ReticleSettings.SavedVars.leftLabelShowColor end )
	-- Use Reticle Visibility
	LAM:AddCheckbox( panel, "leftLabelLinkReticle", "Use Reticle Visibility", "Hide this label when Reticle is hidden.",
		function() return ReticleSettings.SavedVars.leftLabelLinkReticle end,
		function() ReticleSettings.SavedVars.leftLabelLinkReticle = not ReticleSettings.SavedVars.leftLabelLinkReticle end )
	-- Scale
	LAM:AddSlider( panel, "leftLabelScale", "Scale", "", 70, 140, 1,
		function() return ReticleSettings.SavedVars.leftLabelScale end,
		function( scale ) ReticleSettings.SavedVars.leftLabelScale = scale end )
	-- Transparency
	LAM:AddSlider( panel, "leftLabelAlpha", "Transparency", "", 20, 100, 1,
		function() return ReticleSettings.SavedVars.leftLabelAlpha end,
		function( scale ) ReticleSettings.SavedVars.leftLabelAlpha = scale end )
	-- Position
	LAM:AddSlider( panel, "leftLabelPos", "Position", "", -300, -600, -10,
		function() return ReticleSettings.SavedVars.leftLabelPos end,
		function( scale ) ReticleSettings.SavedVars.leftLabelPos = scale end )
	
	-- Save Settings
	--settings = ReticleSettings.SavedVars
	
	-- Create container for custom Reticle
	createCustomReticle()
	
	-- Set Interact Text to alternative position
	if ( ReticleSettings.SavedVars.optInteractTextAltPosition ) then
		ZO_ReticleContainerInteract:ClearAnchors()
		ZO_ReticleContainerInteract:SetAnchor( CENTER, ZO_ReticleContainer, CENTER, 35, 250 )
	end
	
end

function ReticleSettings.WeaponChanged(eventCode, activeWeaponPair, locked)
   
    ActiveWeapon = tostring(activeWeaponPair)

end

function ReticleSettings.GetCustomColor( hex )
	local r, g, b = ReticleSettings.HexToRGB( hex )
	return { r, g, b }
end

-- ***********************************************
-- ** Helper									**
-- ***********************************************

-- Round up decimal
function ReticleSettings.round( val, decimal )
	local exp = decimal and 10^decimal or 1
	return math.ceil( val * exp - 0.5 ) / exp
end

-- Return percentage
function ReticleSettings.getPercent( currentNumber, maxNumber, maxPercent )
	return math.floor( currentNumber / maxNumber * maxPercent )
end

-- Convert RGB to HEX
function ReticleSettings.RGBToHex( r, g, b )
	r = r <= 1 and r >= 0 and r or 0
	g = g <= 1 and g >= 0 and g or 0
	b = b <= 1 and b >= 0 and b or 0
	return string.format( "%02x%02x%02x", r * 255, g * 255, b * 255 )
end

-- Convert HEX to RGB
function ReticleSettings.HexToRGB( hex )
	local rhex, ghex, bhex = string.sub(hex, 1, 2), string.sub(hex, 3, 4), string.sub(hex, 5, 6)
	return tonumber( rhex, 16 )/255, tonumber( ghex, 16 )/255, tonumber( bhex, 16 )/255
end

-- ***********************************************
-- ** Events									**
-- ***********************************************
EVENT_MANAGER:RegisterForEvent("ReticleSettings", EVENT_ADD_ON_LOADED, ReticleSettings.OnInitialized)
EVENT_MANAGER:RegisterForEvent("ActiveWeaponText", EVENT_ACTIVE_WEAPON_PAIR_CHANGED, ReticleSettings.WeaponChanged)